/**
 * @brief LCDキャラクタディスプレイモジュール 制御関数
 * 最終更新日 20XX/XX/XX by valval
 * @note 2020/08/07 OLEにはnewlineの処理がないが、basic.cpp も含めて修正対応を行った
*/
#include <Arduino.h>
#include "LiquidCrystal.h"
#include "SC1602BS.h"

//                RS ENABLE D4 D5 D6 D7
LiquidCrystal lcd(5, 26, 27, 28, 29, 30);       

/**
 * @fn void LCDinit(void)
 * @brief ディスプレイの初期化
 */
void LCDinit(void)
{
    lcd.begin(16, 2);
}
/**
 * @brief コントラスト設定
 * @note LCD表示器にその機能はないため何もしない
 */
void LCDcontrast(int cont)         // コントラスト設定
{
}

/**
 * @brief カーソル移動
 * @param x カーソル移動位置 行
 * @param y カーソル移動位置 桁
 */
void LCDPos(uint8_t x, uint8_t y)
{
    lcd.setCursor(x, y);
}
/**
 * @brief 文字列の出力表示
 * @param text 表示文字列
 */
void LCDprint(uint8_t* text)
{
    lcd.print((char*)text);
}

/**
 * @brief 文字の表示出力
 * @param c 表示出力文字
 */
void LCDputch(uint8_t c)
{
    lcd.write(c);
}

/**
 * @brief 表示のクリア
 */
void LCDcls(void)
{
    lcd.clear();
}

/**
 * @brief LCDの表示のON/OFF指定
 * @param sw   ( 0 で非表示 1で表示）
 */
void LCDisplay(uint8_t sw)
{
    if (sw == 1) {
        lcd.display();
    }
    else {
        lcd.noDisplay();
    }
}

/**
 * @brief カーソル表示指定
 * @param  sw (0 , 1 , 2で点滅 )
 */
void LCDCurs(uint8_t sw)
{
    if (sw==0) {
        lcd.noCursor();
    }
    else if (sw==1) {
        lcd.cursor();
    }
    else if (sw==2) {
        lcd.blink();
    }
    else {
        ;
    }
}

/**
 * @brief 改行表示
 */
void LCDnewline(void)
{
    lcd.print("\n");
}
