/**
 * @file basic.cpp
 * @author your name (you@domain.com)
 * @brief main routine
 * @version 0.1
 * @date 2021-11-23
 * 
 * @copyright Copyright (c) 2012 Tetsuya Suzuki
 * GNU General Public License
 */
#include <Arduino.h>
#include <stdlib.h>
//#include <avr/pgmspace.h>

//*** 環境設定ファイル ****************************
#include "ttconfig.h"          

//*** 外部プロトタイプ宣言 *************************
#include "basic.h"
#include "workmemdef.h"
#include "SO1602AWWB.h"
#include "SC1602BS.h"
#include "files.h"
#include "gpio.h"
#include "IR.h"
#include "sub.h"
#include "device.h"
#include "console.h"
#include "charlib.h"
#include "charinout.h"
#include "sound.h"
#include "Event.h"
#include "intercode.h"
#include "errcode.h"
#include "keyword.h"
#include "errmsg.h"
#include "ctrlcode.h"
#include "getparam.h"
#include "gotogosub.h"
#include "iprint.h"
#include "basic.h"

//*** 機器依存識別 ********************************
#define STR_EDITION_ \
"TOYOSHIKI TINY BASIC\n" \
"GR-ADZUKI(RL78-13) Version 0.07f \n"\
"based ARDUINO Extended version 0.07 \n"

//*** キーワード数定義 *****************************

#include "table.h"

//*** エラー発生情報保持変数 ************************
uint8_t err;             // エラーコード
int16_t errorLine = -1;  // 直前のエラー発生行番号

//*** インタプリタ用グローバル変数 *******************

uint8_t ibuf[SIZE_IBUF];     //!< 中間コード変換バッファ
int16_t var[26];             //!< 変数領域（A-Z×2バイト)
int16_t arr[SIZE_ARRY];      //!< 配列変数領域
uint8_t listbuf[SIZE_LIST];  //!< プログラム領域
uint8_t* clp;                //!< カレント行先頭ポインタ
uint8_t* cip;                //!< インタプリタ中間コード参照位置
uint8_t* gstk[SIZE_GSTK];    //!< GOSUB スタック
uint8_t gstki;               //!< GOSUB スタック インデックス
uint8_t* lstk[SIZE_LSTK];    //!< FOR スタック
uint8_t lstki;               //!< FOR 市タック インデックスtoktoi()
uint8_t val_if = 1;          //!< if文判定結果
uint8_t prevPressKey = 0;    //!< 直前入力キーの値(INKEY()、[ESC]中断キー競合防止用)

//*** 関数の定義 ***********************************
/**
 * @fn int16_t getnum()
 * @brief 数値の入力
 * @return int16_t 取得した数値
 * @note 2021/6/18 Called from iinput() 外部からの呼び出しなし
 */
int16_t getnum() {
  int16_t value = 0 , tmp = 0; // 値と計算過程の値
  char c;             // 文字
  uint8_t len =  0;   // 文字数
  uint8_t sign = 0;   // 負号
  uint8_t str[7];     // 入力文字バッファ

  // 文字入力ループ
  for(;;) {
    c = c_getch();
    if (c == CHAR_CR && len) {
        // 1文字以上入力で[ENTER]確定
        break;
    } else if (c == CHAR_CTRL_C || c == CHAR_ESCAPE) {
        // [CTRL+C] or [ESC]で入力中止
        err = ERR_CTR_C;
        break;
    } else if (((c == CHAR_BACKSPACE) || (c == CHAR_DEL)) && (len > 0)) {
        //［BS］or [DEL] かつ 1文字以上入力あり
        len--; // 文字数を1減らす
        c_putch(CHAR_BACKSPACE,CDEV_SCREEN); // 文字を消す(1)
        c_putch(' ',CDEV_SCREEN);            // 文字を消す(2)
        c_putch(CHAR_BACKSPACE,CDEV_SCREEN); // 文字を消す(3)
    } else if ((len == 0 && (c == '+' || c == '-')) || (len < 6 && c_isdigit(c))) {
        // 行頭の符号および数字が入力された場合の処理（符号込みで6桁を超えないこと）        
        str[len++] = c; // バッファへ入れて文字数を1増やす
        c_putch(c, CDEV_SCREEN); //表示
    } else {      
      c_beep();
    }
  }
  newline();    // 改行
  str[len] = 0; // 終端を置く
  if (str[0] == '-') {
    sign = 1;   // 負の値
    len = 1;    // 数字列はstr[1]以降    
  } else if (str[0] == '+') {
    len = 1;    // 数字列はstr[1]以降
  } else {
    len = 0;    // 数字列はstr[0]以降    
  }
 
  while (str[len]) {  // 終端でなければ繰り返す
    tmp = 10 * value + str[len++] - '0'; //数字を値に変換
    if (value > tmp) { // もし計算過程の値が前回より小さければ
      err = ERR_VOF;   // オーバーフローを記録
    }
    value = tmp;       // 計算過程の値を記録
  }

  if (sign)        // もし負の値なら
    return -value; // 負の値に変換して持ち帰る
  return value;    // 値を持ち帰る
}

/**
 * @brief エラーメッセージ出力
 * @param flgCmd プログラム実行時 false、コマンド実行時 true
 * @note 探した範囲で error(true)のCALLはない。修正時は flgcmdに注意
 * @todo CALL側 error(false) を error()に関数を void error()に修正
 */
void error(void) {  
  c_show_curs(0); // メッセージ表示中はカーソル非表示

  //「OK」でない場合、エラーメッセージを出力する
  if (err) { 
  
    // もしプログラムの実行中のエラーなら発生行の内容を付加して出力する
    //（cip がプログラム領域の中にあり、clpが末尾ではない場合）

    if (cip >= listbuf && cip < listbuf + SIZE_LIST && *clp ) {

      // エラーメッセージを表示      
      c_puts((const char*)pgm_read_word(&errmsg[err]));
      c_puts((const char*)F(" in "));
      errorLine = getlineno(clp);
      putnum(errorLine, 0); // 行番号を調べて表示
      newline();

      // リストの該当行を表示
      putnum(getlineno(clp), 0);
      c_putch(' ', CDEV_SCREEN);
      putlist(clp + 3, 0);          
      newline();

    // ダイレクトにコマンドを実行しば場合、エラーメッセージのみ出力
    } else {
      errorLine = -1;
      c_puts((const char*)pgm_read_word(&errmsg[err]));
      newline();
    }
  }
  
  //「OK」を表示
  c_puts((const char*)pgm_read_word(&errmsg[0])); 
  newline();
  c_show_curs(1); // カーソル再表示
  
  // エラー番号をクリア
  err = 0;                                          
}

//*****************************
// テキスト・中間コード変換用関数 *
//*****************************

/**
 * @brief キーワード検索
 * @return 該当なし -1 見つかった キーワードコード
 * @note 外部参照なし 多分変更不要
 */
int16_t lookup(char* str, uint8_t len) {
  int16_t fd_id;
  int16_t prv_fd_id = -1;
  uint8_t fd_len,prv_len;
  char kwtbl_str[16]; // コマンド比較用
  
  for (uint8_t j = 1; j <= len; j++) {
    fd_id = -1;
    for (uint16_t i = 0; i < SIZE_KWTBL; i++) {
      strcpy_P(kwtbl_str, (char*)pgm_read_word(&(kwtbl[i]))); 
      if (!strncasecmp(kwtbl_str, str, j)) {
        fd_id = i;
        fd_len = j;        
        break;
      }
    }
    if (fd_id >= 0) {
      prv_fd_id = fd_id;
      prv_len = fd_len;
    } else {
      break;
    }
  }
  if (prv_fd_id >= 0) {
    prv_fd_id = -1;
    for (uint16_t i = 0; i < SIZE_KWTBL; i++) {
      strcpy_P(kwtbl_str, (char*)pgm_read_word(&(kwtbl[i]))); 
      if ( (strlen(kwtbl_str) == prv_len) && !strncasecmp(kwtbl_str, str, prv_len) ) {
        prv_fd_id = i;
        break;
      }
    }
  }
  return prv_fd_id;
}

/**
 * @brief トークンを中間コードに変換
 * @return 戻り値 中間コードの並びの長さ or 0
 * @note 外部参照なし 多分変更不要
 */
uint8_t toktoi() {
  uint8_t i;              // ループカウンタ（一部の処理で中間コードに相当）
  int16_t key;            // 中間コード
  uint8_t len = 0;        // 中間コード領域のサイズ
  char* ptok;             // ひとつの単語の内部を指すポインタ
  char c;                 // 文字列の括りに使われている文字（「"」または「'」）
  uint16_t value;         // 定数
  uint32_t tmp;           // 変換過程の定数
  uint8_t  cnt;           // 桁数
  uint8_t  spcnt = 0;     // 先頭スペースカウント
  char* s = (char*)lbuf;       // 文字列バッファの内部を指すポインタ    
  while (*s) {                 // 文字列1行分の終端まで繰り返す
    while (c_isspace(*s))s++;  // 空白を読み飛ばす
    key = lookup(s, strlen(s));// キーワードを切り出し、中間コードを取得
    if (key >= 0) {    
      // 有効なキーワードあり
      if (len >= SIZE_IBUF - 1) {      // もし中間コード領域の容量チェック
        err = ERR_IBUFOF;              // 容量オーバー、エラー番号をセット
        return 0;
      }
      // 中間コードを格納
      ibuf[len++] = key;
      // キーワード長分、取り出し位置移動
      s+= strlen_P((char*)pgm_read_word(&(kwtbl[key])));
    }
    
    if (key == I_DOLLAR) {
      // 中間コードが16進数：$XXXX
      if (isHexadecimalDigit(*s)) {         // もし文字が16進数文字なら
        value = 0;                          // 定数をクリア
        cnt = 0;                            // 桁数
        do { //次の処理をやってみる
          value = (value<<4) | hex2value(*s++); // 数字を値に変換
          cnt++;
        } while (isHexadecimalDigit(*s));   // 16進数文字がある限り繰り返す

        if (cnt > 4) {      // 桁溢れチェック
          err = ERR_VOF;     // エラー番号オバーフローをセット
          return 0;          // 0を持ち帰る
        }
  
        if (len >= SIZE_IBUF - 3) { // もし中間コードが長すぎたら
          err = ERR_IBUFOF;         // エラー番号をセット
          return 0;                 // 0を持ち帰る
        }

        len--;    // I_DALLARを置き換えるために格納位置を移動
        ibuf[len++] = I_HEXNUM;  //中間コードを記録
        ibuf[len++] = value & 255; //定数の下位バイトを記録
        ibuf[len++] = value >> 8;  //定数の上位バイトを記録
      }      
    }

    // 2進数の変換を試みる $XXXX
    if (key == I_APOST) {
      if ( *s == '0'|| *s == '1' ) {    // もし文字が2進数文字なら
        value = 0;                      // 定数をクリア
        cnt = 0;                        // 桁数
        do { //次の処理をやってみる
          value = (value<<1) + (*s++)-'0' ; // 数字を値に変換
          cnt++;
        } while ( *s == '0'|| *s == '1' ); //16進数文字がある限り繰り返す

        if (cnt > 16) {      // 桁溢れチェック
          err = ERR_VOF;      // エラー番号オバーフローをセット
          return 0;
        }
  
        if (len >= SIZE_IBUF - 3) { // もし中間コードが長すぎたら
          err = ERR_IBUFOF;         // エラー番号をセット
          return 0;
        }
        len--;    // I_APOSTを置き換えるために格納位置を移動
        ibuf[len++] = I_BINNUM;  // 中間コードを記録
        ibuf[len++] = value & 255; // 定数の下位バイトを記録
        ibuf[len++] = value >> 8;  // 定数の上位バイトを記録
      }      
    }
    
    // コメントへの変換を試みる
    if(key == I_REM || key == I_SQUOT) {       // もし中間コードがI_REMなら
      while (c_isspace(*s)) s++; //空白を読み飛ばす
      ptok = s; //コメントの先頭を指す

      for (i = 0; *ptok++; i++);      // コメントの文字数を得る
      if (len >= SIZE_IBUF - 2 - i) { // もし中間コードが長すぎたら
        err = ERR_IBUFOF;             // エラー番号をセット
        return 0;
      }

      ibuf[len++] = i;      // コメントの文字数を記録
      while (i--) {         // コメントの文字数だけ繰り返す
        ibuf[len++] = *s++; // コメントを記録
      }
      break; //文字列の処理を打ち切る（終端の処理へ進む）
    }

   if (key >= 0)  // もしすでにキーワードで変換に成功していたら以降はスキップ
     continue;

    //10進数定数への変換を試みる
    ptok = s;                            // 単語の先頭を指す
    if (c_isdigit(*ptok)) {              // もし文字が数字なら
      value = 0;                         // 定数をクリア
      tmp = 0;                           // 変換過程の定数をクリア
      do { //次の処理をやってみる
        tmp = 10 * value + *ptok++ - '0'; // 数字を値に変換
        if (tmp > 32768) {                // もし32768より大きければ
          err = ERR_VOF;                  // エラー番号をセット
          return 0;
        }      
        value = tmp; //0を持ち帰る
      } while (c_isdigit(*ptok)); //文字が数字である限り繰り返す

      if ( (tmp == 32768) && (len > 0) && (ibuf[len-1] != I_MINUS)) {
        err = ERR_VOF;                  // valueが32768のオーバーフローエラー
        return 0;
      }

      if (len >= SIZE_IBUF - 3) { // もし中間コードが長すぎたら
        err = ERR_IBUFOF;         // バッファオーバー
        return 0;
      }

      s = ptok; // 文字列の処理ずみの部分を詰める
      ibuf[len++] = I_NUM;       // 中間コードを記録
      ibuf[len++] = value & 255; // 定数の下位バイトを記録
      ibuf[len++] = value >> 8;  // 定数の上位バイトを記録

    //文字列への変換を試みる
    } else if (*s == '\"') { //もし文字が「"」なら
      c = *s++; //「"」か「'」を記憶して次の文字へ進む
      ptok = s; //文字列の先頭を指す
      //文字列の文字数を得る
      for (i = 0; (*ptok != c) && c_isprint(*ptok); i++)
        ptok++;
      if (len >= SIZE_IBUF - 1 - i) { // もし中間コードが長すぎたら
        err = ERR_IBUFOF;             // エラー番号をセット
        return 0;
      }
      ibuf[len++] = I_STR;  // 中間コードを記録
      ibuf[len++] = i;      // 文字列の文字数を記録
      while (i--) {         // 文字列の文字数だけ繰り返す
        ibuf[len++] = *s++; // 文字列を記録
      }
      if (*s == c) s++; //もし文字が「"」なら次の文字へ進む

    //変数への変換を試みる    
    } else if (c_isalpha(*ptok)) { // もし文字がアルファベットなら
      if (len >= SIZE_IBUF - 2) {  // もし中間コードが長すぎたら
        err = ERR_IBUFOF;          // バッファオーバー
        return 0;
      }
    
      //もし変数が3個並んだら
      if (len >= 4 && ibuf[len - 2] == I_VAR && ibuf[len - 4] == I_VAR) {
        err = ERR_SYNTAX;  // エラー番号をセット
        return 0;
      }

      ibuf[len++] = I_VAR;                  // 中間コードを記録
      ibuf[len++] = c_toupper(*ptok) - 'A'; // 変数番号を記録
      s++;                                  // 次の文字へ進む

    // どれにも当てはまらなかった場合    
    } else {
      err = ERR_SYNTAX; // エラー番号をセット
      return 0;
    }
  } // 文字列1行分の終端まで繰り返すの末尾

  ibuf[len++] = I_EOL; // 文字列1行分の終端を記録
  return len;          // 中間コードの長さを持ち帰る
}

/** 
 * @brief 行番号から行インデックスを取得する
 * @param lineno 指定行番号
 * @return 行インデックスを返す
 */
uint16_t getlineIndex(uint16_t lineno) {
  uint8_t *lp; // ポインタ
  uint16_t index = 0;  
  uint16_t rc = 32767;
  for (lp = listbuf; *lp; lp += *lp) {           // 先頭から末尾まで繰り返す
    if ((uint16_t)getlineno(lp) >= lineno) {     // もし指定の行番号以上なら
      rc = index;
      break;                                     // 繰り返しを打ち切る
    }
    index++;
  }
  return rc; 
}

/**
 * @brief プログラム行数を取得する
 * @return プログラム行数を返す
 * @note パラメータなしで1箇所しか呼ばれてないのでパラメータなし関数に変更
 */
//uint16_t countLines(int16_t st=0, int16_t ed=32767) {
uint16_t countLines() {
  int16_t st = 0;       // 開始行番号
  int16_t ed = 32767;     // 修了行番号
  uint8_t *lp; //ポインタ
  uint16_t cnt = 0;  
  int16_t  lineno;
  for (lp = listbuf; *lp; lp += *lp)  {
    lineno = getlineno(lp);
    if (lineno < 0)
      break;
    if ( (lineno >= st) && (lineno <= ed)) 
      cnt++;
  }
  return cnt;   
}

/**
 * @brief 指定行の削除
 * @param no : 指定された行番号
 * @return 1: 正常終了  0:削除対象行なし
 */
bool dellist(int16_t no) {
  uint8_t *lp;      // 削除対象位置
  uint8_t *p1, *p2; // 移動先と移動元
  uint8_t  len;     // 移動の長さ

  lp = getlp(no);   // 削除位置ポインタを取得
  if (getlineno(lp) == no) {
    p1 = lp;                              // p1を挿入位置に設定
    p2 = p1 + *p1;                        // p2を次の行に設定
    while ((len = *p2) != 0) {            // 次の行の長さが0でなければ繰り返す
      while (len--)                       // 次の行の長さだけ繰り返す
        *p1++ = *p2++;                    // 前へ詰める
    }
    *p1 = 0; // リストの末尾に0を置く
    return false;
  }
  return true;
}

/**
 * @brief プログラムリスト（中間コード領域）に行の挿入
 * @note  ibuf 挿入するプログラムの文字列ポインタ
 */
void inslist() {
  uint8_t *insp;    // listbuf領域内挿入位置
  uint8_t *p1, *p2; // 移動先と移動元
  uint16_t len;     // 移動の長さ

  // 領域容量チャック
  if (getsize() < *ibuf) {
    err = ERR_LBUFOF;   // エラー番号をセット
    return;
  }

  // 挿入位置を取得
  insp = getlp(getlineno(ibuf)); 
  
  // 同じ行番号の行が存在したらとりあえず削除
  dellist(getlineno(ibuf));

  // 行番号だけが入力された場合はここで終わる
  if (*ibuf == 4) // もし長さが4（行番号のみ）なら
    return;       // 終了する

  // 挿入のためのスペースを空ける
  for (p1 = insp; *p1; p1 += *p1); // p1をリストの末尾へ移動
  len = p1 - insp + 1;             // 移動する幅を計算
  p2 = p1 + *ibuf;                 // p2を末尾より1行の長さだけ後ろに設定
  while (len--)                    // 移動する幅だけ繰り返す
    *p2-- = *p1--;                 // 後ろへズラす

  // 行を転送する
  len = *ibuf; // 中間コードの長さを設定
  p1 = insp;   // 転送先を設定
  p2 = ibuf;   // 転送元を設定
  while (len--) // 中間コードの長さだけ繰り返す
    *p1++ = *p2++; // 転送
}

/**
 * @brief 変数代入式の評価 
 * @details  例：A=値|式
 */
void ivar() {
  int16_t value;  // 値
  int16_t index;  // 変数番号

  index = *cip++; // 変数番号を取得して次へ進む

  // 「=」のチェック
  if (*cip != I_EQ) {
    err = ERR_VWOEQ;  // エラー番号をセット
    return;
  } 
  cip++;

  if (*cip == I_STR) {
    // 代入対象が文字列定数の場合、文字列の仮想アドレスを代入
    cip++;
    value = (int16_t)((uint32_t)cip - (uint32_t)listbuf + V_PRG_TOP);
    cip += *cip+1;
  } else {
    // そうでない場合、式の評価(整数値)と代入
    value = iexp(); // 式の値を取得
    if (err)        // もしエラーが生じたら
      return;
  }
  var[index] = value; //変数へ代入
}

/**
 * @brief 配列への代入式の評価
 */
void iarray() {
  int16_t value; // 値
  int16_t index; // 配列の添え字

  index = getparamValue(); // 配列の添え字を取得
  if (err)
    return;

  // 添え字範囲チェック
  if (index >= SIZE_ARRY || index < 0 ) {
    err = ERR_SOR;   // エラー番号をセット
    return;
  }

  // 「=」のチェック
  if (*cip != I_EQ) {
    err = ERR_VWOEQ; // エラー番号をセット
    return;
  }

  // 連続代入の処理 例: @(n)=1,2,3,4,5 
  do {
    cip++; 
    if (*cip == I_STR) {
      // 代入対象が文字列定数の場合、文字列の仮想アドレスを代入   
      cip++;
      value = (int16_t)((uint32_t)cip - (uint32_t)listbuf + V_PRG_TOP);
      cip += *cip+1;
    } else {
      // そうでない場合、式の評価(整数値)と代入    
      value = iexp(); // 式の値を取得
      if (err)
        return;
    }
    
    if (index >= SIZE_ARRY) { // もし添え字が上限を超えたら
      err = ERR_SOR;          // エラー番号をセット
      return; 
    }
    
    arr[index] = value;       // 配列へ代入
    index++;
  } while(*cip == I_COMMA);
} 

/**
 * @brief ELSE中間コードをサーチする
 * @param p 中間コードプログラムポインタ
 * @return uint8_t* NULL:見つからない NULL以外 I_ELSEの次のポインタ
 */
uint8_t* getELSEptr(uint8_t* p) {
  uint8_t* rc = NULL;
  uint8_t* lp;

  for (lp = p; *lp != I_EOL ; ) {
    switch(*lp) {
    case I_IF:      // IF命令
      goto DONE;
      break;
    case I_ELSE:    // ELSE命令
      rc = lp+1;
      goto DONE;
      break;
    case I_STR:     // 文字列
      lp += lp[1]+1;            
      break;
    case I_NUM:     // 定数
    case I_HEXNUM: 
    case I_BINNUM:
      lp+=3;        // 整数2バイト+中間コード1バイト分移動
      break;
    case I_VAR:     // 変数
      lp+=2;        // 変数名
      break;
    default:        // その他
      lp++;
    }
  }

DONE:
  return rc;
}

/**
 * @brief INPUT handler
 */
void iinput() {
  int16_t value;          // 値
  int16_t index;          // 配列の添え字or変数番号
  uint8_t i;              // 文字数
  uint8_t prompt;         // プロンプト表示フラグ
  int16_t ofvalue;        // オーバーフロー時の設定値
  uint8_t flgofset =0;    // オーバーフロ時の設定値指定あり

  c_show_curs(1);
  prompt = 1;          // まだプロンプトを表示していない

  // プロンプトが指定された場合の処理
  if(*cip == I_STR){   // もし中間コードが文字列なら
    cip++;             // 中間コードポインタを次へ進める
    i = *cip++;        // 文字数を取得
    while (i--)        // 文字数だけ繰り返す
      c_putch(*cip++, CDEV_SCREEN); // 文字を表示
    prompt = 0;        // プロンプトを表示した

    if (*cip != I_COMMA) {
      err = ERR_SYNTAX;
      goto DONE;
    }
    cip++;
  }

  // 値を入力する処理
  switch (*cip++) {       // 中間コードで分岐
  case I_VAR:             // 変数の場合
    index = *cip;         // 変数番号の取得
    cip++;
   
    // オーバーフロー時の設定値
    if (*cip == I_COMMA) {
      cip++;
      ofvalue = iexp();
      if (err) {
        goto DONE;
      }
      flgofset = 1;
    }
    
    if (prompt) {          // もしまだプロンプトを表示していなければ
      c_putch('A'+index, CDEV_SCREEN);  // 変数名を表示
      c_putch(':', CDEV_SCREEN);        //「:」を表示
    }
    
    value = getnum();     // 値を入力
    if (err) {            // もしエラーが生じたら
      if (err == ERR_VOF && flgofset) {
        err = ERR_OK;
        value = ofvalue;
      } else {
        return;            // 終了
      }
    }
    var[index] = value;  // 変数へ代入
    break;               // 打ち切る

  case I_ARRAY: // 配列の場合
    index = getparamValue();       // 配列の添え字を取得
    if (err)                  // もしエラーが生じたら
      goto DONE;

    if (index >= SIZE_ARRY) { // もし添え字が上限を超えたら
      err = ERR_SOR;          // エラー番号をセット
      goto DONE;
    }

    // オーバーフロー時の設定値
    if (*cip == I_COMMA) {
      cip++;
      ofvalue = iexp();
      if (err) {
        goto DONE;
      }
      flgofset = 1;
    }

    if (prompt) { // もしまだプロンプトを表示していなければ
      c_puts((const char*)F("@("));     //「@(」を表示
      putnum(index, 0);                   // 添え字を表示
      c_puts((const char*)F("):"));     //「):」を表示
    }
    value = getnum(); // 値を入力
    if (err) {           // もしエラーが生じたら
      if (err == ERR_VOF && flgofset) {
        err = ERR_OK;
        value = ofvalue;
      } else {
        goto DONE;
      }
    }
    arr[index] = value; // 配列へ代入
    break;              // 打ち切る

  default: // 以上のいずれにも該当しなかった場合
    err = ERR_SYNTAX; // エラー番号をセット
    goto DONE;
  } // 中間コードで分岐の末尾
DONE:  
  c_show_curs(0);
}

/**
 * @brief LET handler
 */
void ilet() {
  if (*cip == I_VAR) {
    // 変数の場合
    cip++; 
    ivar();  // 変数への代入を実行    
  } else if  (*cip == I_ARRAY) {
   // 配列の場合
    cip++; 
    iarray(); // 配列への代入を実行    
  } else {
    err = ERR_LETWOV; //エラー番号をセット    
  }
}
/**
 * @brief I_EOLに到達するまでSKIPする
 */
void iskip() {
  while (*cip != I_EOL) // I_EOLに達するまで繰り返す
    cip++;              // 中間コードポインタを次へ進める
}

/**
 * @brief END処理
 */
void iend() {
  while (*clp)    // 行の終端まで繰り返す
    clp += *clp;  // 行ポインタを次へ進める
}

/**
 * @brief IF処理
 */
void iif() {
  int16_t  condition;   // IF文の条件値
  uint8_t* newip;       // ELSE文以降の処理対象ポインタ

  condition = iexp(); // 真偽を取得
  if (err) {          // もしエラーが生じたら
    err = ERR_IFWOC;  // エラー番号をセット
    return;
  }
  val_if = condition; // 判定結果を保持
  if (condition) {    // もし真なら
    return;
  } else { 
    // 偽の場合の処理
    // ELSEがあるかチェックする
    // もしELSEより先にIFが見つかったらELSE無しとする
    // ELSE文が無い場合の処理はREMと同じ
    newip = getELSEptr(cip);
    if (newip != NULL) {
      cip = newip;
      return;
    }
    iskip(); // ELSE以降をスキップ
  }
}

/**
 * @brief 単独ELSE
 */
void ielse() {
  if(val_if) {
    // 直前のIF文の真偽が真の場合、ELSE以降をスキップ   
    iskip(); 
  }
}

/**
 * @brief ラベルをスキップ 
 */
inline void ilabel() {
   cip+= *cip+1;   
}

/**
 * @brief RETURN処理
 */
void ireturn() {
  if (gstki < 2) {     // もしGOSUBスタックが空なら
    err = ERR_GSTKUF;  // エラー番号をセット
    return; 
  }
  cip = gstk[--gstki]; // 行ポインタを復帰
  clp = gstk[--gstki]; // 中間コードポインタを復帰
  return;  
}

/**
 * @brief FOR処理
 */
void ifor() {
  int16_t index, vto, vstep; // FOR文の変数番号、終了値、増分
  
  // 変数名を取得して開始値を代入（例I=1）
  if (*cip++ != I_VAR) { // もし変数がなかったら
    err = ERR_FORWOV;    // エラー番号をセット
    return;
  }
  index = *cip; // 変数名を取得
  ivar();       // 代入文を実行
  if (err)      // もしエラーが生じたら
    return;

  // 終了値を取得（例TO 5）
  if (*cip == I_TO) { // もしTOだったら
    cip++;             // 中間コードポインタを次へ進める
    vto = iexp();      // 終了値を取得
  } else {             // TOではなかったら
    err = ERR_FORWOTO; //エラー番号をセット
    return;
  }

  // 増分を取得（例STEP 1）
  if (*cip == I_STEP) { // もしSTEPだったら
    cip++;              // 中間コードポインタを次へ進める
    vstep = iexp();     // 増分を取得
  } else                // STEPではなかったら
    vstep = 1;          // 増分を1に設定

  // もし変数がオーバーフローする見込みなら
  if (((vstep < 0) && (-32767 - vstep > vto)) ||
    ((vstep > 0) && (32767 - vstep < vto))){
    err = ERR_VOF; //エラー番号をセット
    return;
  }

  // 繰り返し条件を退避
  if (lstki > SIZE_LSTK - 5) { // もしFORスタックがいっぱいなら
    err = ERR_LSTKOF;          // エラー番号をセット
    return;
  }
  lstk[lstki++] = clp; // 行ポインタを退避
  lstk[lstki++] = cip; // 中間コードポインタを退避

  // FORスタックに終了値、増分、変数名を退避
  // Special thanks hardyboy
  lstk[lstki++] = (uint8_t*)(uintptr_t)vto;
  lstk[lstki++] = (uint8_t*)(uintptr_t)vstep;
  lstk[lstki++] = (uint8_t*)(uintptr_t)index;  
}

/**
 * @brief NEXT処理
 */
void inext() {
  int16_t index, vto, vstep; // FOR文の変数番号、終了値、増分

  if (lstki < 5) {    // もしFORスタックが空なら
    err = ERR_LSTKUF; // エラー番号をセット
    return;
  }

  /**
   * @brief 変数名を復帰
   */
  index = (int16_t)(uintptr_t)lstk[lstki - 1]; // 変数名を復帰
  if (*cip++ != I_VAR) {                       // もしNEXTの後ろに変数がなかったら
    cip--;
  } else {
    if (*cip++ != index) { // もし復帰した変数名と一致しなかったら
      err = ERR_NEXTUM;    // エラー番号をセット
      return;
    }
  }
  vstep = (int16_t)(uintptr_t)lstk[lstki - 2]; // 増分を復帰
  var[index] += vstep;                         // 変数の値を最新の開始値に更新
  vto = (short)(uintptr_t)lstk[lstki - 3];     // 終了値を復帰

  // もし変数の値が終了値を超えていたら
  if (((vstep < 0) && (var[index] < vto)) ||
    ((vstep > 0) && (var[index] > vto))) {
    lstki -= 5;  // FORスタックを1ネスト分戻す
    return;
  }

  // 開始値が終了値を超えていなかった場合
  cip = lstk[lstki - 4]; //行ポインタを復帰
  clp = lstk[lstki - 5]; //中間コードポインタを復帰
}

/**
 * @brief LISTコマンド
 * @param devno デバイス番号 0:メインスクリーン 1:シリアル 2:グラフィック 3:、メモリー 4:ファイル
 */
void ilist(uint8_t devno=0) {
  int16_t lineno = 0;          // 表示開始行番号
  int16_t endlineno = 32767;   // 表示終了行番号
  int16_t prnlineno;           // 出力対象行番号
  uint8_t *bak_clp;            // バックアップ
  
  //表示開始行番号の設定
  if (*cip != I_EOL && *cip != I_COLON) {
    // 引数あり
    if (getParamRange(&lineno,0,32767,false)) return;                // 表示開始行番号
    if (*cip == I_COMMA) {
      cip++;                                                   // カンマをスキップ
      if (getParamRange(&endlineno,lineno,32767,false)) return;      // 表示終了行番号
     }
   }

  bak_clp = clp;
  clp = getlp(lineno);         // 行ポインタを表示開始行番号へ進める

  //リストを表示する
  while (*clp) {               // 行ポインタが末尾を指すまで繰り返す
    if (isBreak())
      return;  //強制的な中断の判定
   
    prnlineno = getlineno(clp);// 行番号取得
    if (prnlineno > endlineno) // 表示終了行番号に達したら抜ける
       break; 
    putnum(prnlineno, 0,devno);// 行番号を表示
    c_putch(' ',devno);        // 空白を入れる
    putlist(clp + 3,devno);    // 行番号より後ろを文字列に変換して表示
    if (err)                   // もしエラーが生じたら
      break;                   // 繰り返しを打ち切る
    newline_dev(devno);            // 改行
    clp += *clp;               // 行ポインタを次の行へ進める
  }
  clp = bak_clp;
}

/**
 * @brief RENUME [開始番号[,増分]]
 * @note ※引数には、符号無し10進数定数のみ指定可能
 */
void irenum() {
  int16_t startLineNo = 10;   // 開始行番号
  int16_t increase = 10;      // 増分
  uint8_t* ptr;               // プログラム領域参照ポインタ
  uint8_t  len;               // 行長さ
  uint8_t  i;                 // 中間コード参照位置
  uint16_t newnum;            // 新しい行番号
  uint16_t num;               // 現在の行番号
  uint16_t index;             // 行インデックス
  uint16_t cnt;               // プログラム行数
  uint8_t *bak_clp;           // バックアップ
  
  // 開始行番号、増分引数チェック
  if (*cip != I_EOL && *cip != I_COLON) {
    // 引数あり
    if (getParamRange(&startLineNo,1,32767,false)) return;     // 開始行番号
    if (*cip == I_COMMA) {
      cip++;                                             // カンマをスキップ
      if (getParamRange(&increase,1,32767,false)) return;      // 増分
     }
  }

  // 引数の有効性チェック
  cnt = countLines()-1;
  if ( (startLineNo <= 0 || increase <= 0) ||
       (startLineNo + increase * cnt > 32767) ) {
    err = ERR_VALUE;
    return;   
  }

  bak_clp = clp;
  // ブログラム中のGOTOの飛び先行番号を付け直す
  for (clp = listbuf; *clp ; clp += *clp) {
     ptr = clp;
     len = *ptr;
     ptr++;
     i=0;
     // 行内検索
     while( i < len-1 ) {
        switch(ptr[i]) {
        case I_GOTO:  // GOTO命令
        case I_GOSUB: // GOSUB命令
          i++;
          if (ptr[i] == I_NUM) {
            num = getlineno(&ptr[i]);    // 現在の行番号を取得する
            index = getlineIndex(num);   // 行番号の行インデックスを取得する
            if (index == 32767) {
               // 該当する行が見つからないため、変更は行わない
               i+=3;
               continue;
            } else {
               // とび先行番号を付け替える
               newnum = startLineNo + increase*index;
               ptr[i+2] = newnum>>8;
               ptr[i+1] = newnum&0xff;
               i+=3;
               continue;
            }
          } 
          break;
      case I_STR:  // 文字列
        i++;
        i+=ptr[i]; // 文字列長分移動
        break;
      case I_NUM:  // 定数
      case I_HEXNUM:
      case I_BINNUM: 
        i+=3;      // 整数2バイト+中間コード1バイト分移動
        break;
      case I_VAR:  // 変数
        i+=2;      // 変数名
        break;
      default:     // その他
        i++;
        break;
      }
    }
  }
  
  // 各行の行番号の付け替え
  index = 0;
  for ( clp = listbuf; *clp ; clp += *clp ) {
     newnum = startLineNo + increase * index;
     *(clp+1)  = newnum&0xff;
     *(clp+2)  = newnum>>8;
     index++;
  }
  clp = bak_clp;
}
 
/**
 * @brief 指定行の削除 
 * @note DELETE 行番号 / DELETE 開始行番号,終了行番号
 */
void idelete() {
  int16_t sNo;       // 開始行番号
  int16_t eNo;       // 終了行番号
  uint8_t *lp;       // 削除位置ポインタ
  int16_t n;         // 削除対象行

  if ( getParamInt16(&sNo, false) ) return;
  if (*cip == I_COMMA) {
     cip++;
     if ( getParamRange(&eNo, sNo,32767,false) ) return;  
  } else {
     eNo = sNo;
  }
  // 行ポインタを表示開始行番号へ進める
  lp = getlp(sNo);
  for (;;) {
     n = getlineno(lp);
     if (n < 0 || n > eNo)
        break;
     dellist(n);
  }
}

/**
 * @brief NEW コマンド
 */
void inew(void) {
  // 変数と配列の初期化
  memset(var,0,26);
  memset(arr,0,SIZE_ARRY);

  // 実行制御用の初期化
  gstki = 0;     // GOSUBスタックインデクスを0に初期化
  lstki = 0;     // FORスタックインデクスを0に初期化
  *listbuf = 0;  // プログラム保存領域の先頭に末尾の印を置く
  clp = listbuf; // 行ポインタをプログラム保存領域の先頭に設定
}

/**
 * @brief 時間待ち 
 */
void iwait() {
  int16_t tm;
  if ( getParamRange(&tm, 0, 32767, false) ) return;
  delay(tm);
}

/**
 * @brief 次のプログラム実行位置(行の先頭)
 */
void iinfo() {
#if  USE_SYSINFO == 1
char top = 't';
  uint16_t adr = (uint32_t)&top;
  uint8_t* tmp = (uint8_t*)malloc(1);
  uint16_t hadr = (uint16_t)tmp;
  free(tmp);

  // スタック領域先頭アドレスの表示
  c_puts((const char*)F("Stack Top:"));
  putHexnum(adr,4);
  
  // ヒープ領域先頭アドレスの表示
  c_puts((const char*)F("\nHeap Top :"));
  putHexnum(hadr,4);

  // SRAM未使用領域の表示
  c_puts((const char*)F("\nSRAM Free:"));
  putnum((int16_t)(adr-hadr),0);

  // コマンドエントリー数
  c_puts((const char*)F("\nCommand table:"));
  putnum((int16_t)(I_EOL+1),0);

  newline();
#endif  
}

/**
 * @brief 中間コードの実行
 * @return uint8_t* 次のプログラム実行位置(行の先頭)
 */
uint8_t* iexe() {
  err = 0;
  while (*cip != I_EOL) { //行末まで繰り返す

  //強制的な中断の判定
  if (isBreak()) 
    break;

    //中間コードを実行
    switch (*cip++) { //中間コードで分岐
    case I_STR:      ilabel();          break;  // 文字列の場合(ラベル)
    case I_GOTO:     iGotoGosub(MODE_GOTO);  break;  // GOTOの場合
    case I_GOSUB:    iGotoGosub(MODE_GOSUB); break;  // GOSUBの場合
    case I_RETURN:   ireturn();         break;  // RETURNの場合
    case I_FOR:      ifor();            break;  // FORの場合
    case I_NEXT:     inext();           break;  // NEXTの場合
    case I_IF:       iif();             break;  // IFの場合
    case I_ELSE:     ielse();           break;  // 単独のELSEの場合    
    case I_SQUOT:                               // 'の場合
    case I_REM:      iskip();           break;  // REMの場合
    case I_END:      iend();            break;  // ENDの場合
    case I_CLS:      icls();            break;  // CLS
    case I_WAIT:     iwait();           break;  // WAIT
    case I_LOCATE:   ilocate();         break;  // LOCATE
    case I_COLOR:    icolor();          break;  // COLOR
    case I_ATTR:     iattr();           break;  // ATTR
    case I_VAR:      ivar();            break;  // 変数（LETを省略した代入文）
    case I_ARRAY:    iarray();          break;  // 配列（LETを省略した代入文）
    case I_LET:      ilet();            break;  // LET
    case I_QUEST:                               // ？
    case I_PRINT:    iprint(0,0);       break;  // PRINT
    case I_INPUT:    iinput();          break;  // INPUT
    case I_POKE:     ipoke();           break;  // POKEコマンド
    case I_GPIO:     igpio();           break;  // GPIO
    case I_DOUT:     iout();            break;  // OUT
    case I_LED:      iled();            break;  // LED
    case I_POUT:     ipwm();            break;  // PWM 
    case I_SHIFTOUT: ishiftOut();       break;  // ShiftOut
#if USE_SO1602AWWB == 1 && USE_CMD_I2C == 1
    case I_CPRINT:   icprint();         break;  // CPRINT
    case I_CCLS:     iccls();           break;  // CCLS
    case I_CCURS:    iccurs();          break;  // CCURS
    case I_CLOCATE:  iclocate();        break;  // CLOCATE
    case I_CCONS:    iccons();          break;  // CCONS
    case I_CDISP:    icdisp();          break;  // CDISP
#endif
#if USE_RTC_DS3231 == 1 && USE_CMD_I2C == 1 || USE_RL78_RTC == 1
    case I_SETDATE:  isetDate();        break;  // SETDATEコマンド
    case I_GETDATE:  igetDate();        break;  // GETDATEコマンド
    case I_GETTIME:  igetTime();        break;  // GETDATEコマンド
    case I_DATE:     idate();           break;  // DATEコマンド
#endif
#if USE_CMD_VFD == 1
    case I_VMSG:     ivmsg();           break;  // VMSG
    case I_VPUT:     ivput();           break;  // VPUT
    case I_VSCROLL:  ivscroll();        break;  // VSCROLL
    case I_VCLS:     ivcls();           break;  // VCLS
    case I_VBRIGHT:  ivbright();        break;  // VBRIGHT
    case I_VDISPLAY: ivdisplay();       break;  // VDISPLAY文を実行
#endif
#if USE_CMD_TONE == 1          // GR-ADZUKI で追加
    case I_TONE:     itone();           break;  // TONE
    case I_NOTONE:   inotone();         break;  // NOTONE
#endif                         // 
#if USE_CMD_PLAY == 1
    case I_PLAY:      iplay();          break;  // PLAY
    case I_TEMPO:     itempo();         break;  // TEMPO    
#endif
#if USE_NEOPIXEL == 1
    case I_NINIT:     ininit();         break;  // NINIT
    case I_NUPDATE:   inupdate();       break;  // NUPDATE
    case I_NBRIGHT:   inbright();       break;  // NBRIGHT
    case I_NCLS:      incls();          break;  // NCLS
    case I_NSET:      inset();          break;  // NSET
    case I_NPSET:     inpset();         break;  // NPSET        
    case I_NSHIFT:    inshift();        break;  // NSHIFT
#if USE_MISAKIFONT != 0
    case I_NMSG:      inmsg();          break;  // NMSG
#endif
    case I_NLINE:     inLine();         break;  // NLINE
    case I_NSCROLL:   inscroll();       break;  // NSCROLL
#endif
#if USE_EVENT == 1
    case I_ON:        iOnPinTimer();    break;  // ON TIMER| ON PIN
    case I_TIMER:     iTimer();         break;  // TIMER
    case I_PIN:       iPin();           break;  // PIN
#endif
#if USE_SLEEP == 1
    case I_SLEEP:     isleep();         break;  // SLEEP
#endif

#if USE_SC1602BS == 1
    case I_LPRINT:   ilprint();         break;  // LPRINT
    case I_LCLS:     ilcls();           break;  // LCLS
    case I_LCURS:    ilcurs();          break;  // LCURS
    case I_LLOCATE:  illocate();        break;  // LLOCATE
    case I_LCONS:    ilcons();          break;  // LCONS
    case I_LDISP:    ildisp();          break;  // LDISP
#endif

    case I_SYSINFO:   iinfo();          break;  // SYSINFO     
    case I_RENUM: irenum();             break;  // RENUMの場合
    case I_DELETE:idelete();            break;  // DELETE

    case I_NEW:   inew();               break;  // NEW  
    case I_LIST:  ilist();              break;  // LIST
    case I_LOAD:  iLoadSave(MODE_LOAD); break;  // LOAD
    case I_SAVE:  iLoadSave(MODE_SAVE); break;  // SAVE

    case I_ERASE: ierase();   break;  // ERASE
    case I_FILES: ifiles();   break;  // FILES
    case I_FORMAT:iformat();  break;  // FORMAT
    case I_DRIVE: idrive();   break;  // DRIVE

    case I_COLON:     break; // 中間コードが「:」の場合   
      
    default:                 // 以上のいずれにも該当しない場合
     cip--;
     if (*cip >= I_RUN && *cip <= I_DRIVE) {
        err = ERR_COM; // エラー番号をセット
     } else {    
        err = ERR_SYNTAX; //エラー番号をセット
     }
     break;
    } //中間コードで分岐の末尾

    if (err)
      return NULL;
#if USE_EVENT == 1
    doTimerEvent();
    if (err)
      return NULL;
    doExtEvent();
    if (err)
      return NULL;
#endif
  } //行末まで繰り返すの末尾
  return clp + *clp; //次に実行するべき行のポインタを持ち帰る
}

/**
 * @brief RUN コマンド
 */
void irun() {
  uint8_t* lp; // 行ポインタの一時的な記憶場所
  initProgram();
  c_show_curs(0);    // カーソル消去
  while (*clp) {     // 行ポインタが末尾を指すまで繰り返す
    cip = clp + 3;   // 中間コードポインタを行番号の後ろに設定
    lp = iexe();     // 中間コードを実行して次の行の位置を得る
    if (err)         // もしエラーを生じたら      
      break;
    clp = lp;        // 行ポインタを次の行の位置へ移動
  }
  c_show_curs(1);    // カーソル表示
#if USE_EVENT == 1
  clerTimerEvent();
  clerExtEvent();
#endif
}

/**
 * @brief Command Processor (コマンドラインからのコマンド実行)
 * @return uint8_t 実行結果( 1: Error 0:Success)
 */
uint8_t icom() {
  uint8_t rc = 1;
  cip = ibuf;       // 中間コードポインタを中間コードバッファの先頭に設定
  //c_show_curs(0);  
  switch (*cip++) { // 中間コードポインタが指し示す中間コードによって分岐
  case I_RUN:   irun();     break;  // RUN命令
  case I_REM:
  case I_SQUOT:
  case I_OK:    rc = 0;     break; // I_OKの場合
  default: //どれにも該当しない場合
    cip--;
    c_show_curs(0);  
    iexe(); //中間コードを実行
    c_show_curs(1);  
    break; //打ち切る
  }
  //c_show_curs(1);      
  return rc;
}

/**
 * @brief Basic Entry Point
 */
void basic() {
  uint8_t len;     // 中間コードの長さ

  init_console();  // シリアルコンソールの初期設定

#if USE_CMD_VFD == 1 
  VFD_init();  // VFD利用開始
#endif
#if USE_CMD_I2C == 1
  i2c_init();  // I2C利用開始  
#endif  
#if USE_CMD_PLAY == 1
  mml_init();  // MML初期化
#endif
#if USE_SO1602AWWB == 1 && USE_CMD_I2C == 1
  // OLEDキャラクタディスプレイ
  OLEDinit();
#endif
#if USE_EVENT == 1
  initTimerEvent();
  clerExtEvent();
#endif
#if USE_SC1602BS == 1 
  // LCDキャラクタディスプレイ
  LCDinit();
#endif

  inew(); // 実行環境を初期化
  icls(); // 画面クリア

  //起動メッセージ
  c_puts((const char*)F(STR_EDITION_));
  putnum(getsize(),3); c_puts((const char*)F("byte free\n")); //プログラム領域を表示
  error(); //「OK」またはエラーメッセージを表示してエラー番号をクリア
  
  // リセット時に指定PINがHIGHの場合、プログラム自動起動
  if (digitalRead(AutoPin)) {
    // ロードに成功したら、プログラムを実行する
    iLoadSave(MODE_LOAD,true);   // プログラムのロード
    irun();                      // RUN命令を実行
    newline();                   // 改行
    error();                     // エラーメッセージ出力
    err = 0; 
  }

  // 端末から1行を入力して実行
  c_show_curs(1);
  for (;;) { //無限ループ
    c_putch('>',CDEV_SCREEN); // プロンプトを表示
    c_gets();     // ラインエディタ（lbufに格納）
    if(!strlen((char*)lbuf))  continue;

    // 1行の文字列を中間コードの並びに変換
    len = toktoi(); // 文字列を中間コードに変換して長さを取得（ibufに買機能）
    if (err) {      // もしエラーが発生したら
      clp = NULL;
      error();      // エラーメッセージを表示してエラー番号をクリア
      continue;     // 繰り返しの先頭へ戻ってやり直し
    }

    // 中間コードの並びがプログラムと判断される場合
    if (*ibuf == I_NUM) { // 先頭が行番号の場合、
      *ibuf = len;        // 中間コードバッファの先頭を長さに書き換える
      inslist();          // 中間コードの1行をリストへ挿入
      if (err)            // 挿入エラー
        error();          // エラーメッセージを表示してエラー番号をクリア
      continue;           // 繰り返しの先頭へ戻ってやり直し
    } else {
    // 中間コードの並びが命令と判断される場合
      if (icom())           // 実行する
        error();            // エラーメッセージを表示してエラー番号をクリア       
    }
  } //無限ループの末尾
}
