/**
 * @file gotogosub.cpp
 * @author your name (you@domain.com)
 * @brief GOTO/GOSUB　関係の関数群
 * @version 0.1
 * @date 2021-11-23
 * 
 * @copyright Copyright (c) 2021
 * 
 */
#include "Arduino.h"
#include "getparam.h"
#include "console.h"
#include "errcode.h"
#include "intercode.h"
#include "basic.h"
#include "gotogosub.h"

/** 
 * @brief ラベルでリストポインタを取得する。
 * @param pLabel 
 * @return リストポインタを返す。見つからないNULLを返す
 * @note pLabelは [I_STR][長さ][ラベル名] であること
 */
uint8_t* getlpByLabel(uint8_t* pLabel) {
  uint8_t *lp; //ポインタ
  uint8_t len;
  pLabel++;
  len = *pLabel; // 長さ取得
  pLabel++;      // ラベル格納位置
  
  for (lp = listbuf; *lp; lp += *lp)  { //先頭から末尾まで繰り返す
    if ( *(lp+3) == I_STR ) {
       if (len == *(lp+4)) {
           if (strncmp((char*)pLabel, (char*)(lp+5), len) == 0) {
              return lp;
           }
       }
    }  
  }
  return NULL;
}

/**
 * @brief GOTO/GOSUB ジャンプ先リストポインタ取得
 * @return uint8_t* ジャンプ先リストポインタを返す
 */
uint8_t* getJumplp() {  
  int16_t lineno;    // 行番号
  uint8_t* lp;       // 飛び先行ポインタ  
  if (*cip == I_STR) { 
    // 引数がラベル
    lp = getlpByLabel(cip);
    if (lp == NULL) {
      // 飛び先ラベルが見つからない
      err = ERR_ULN;
      return 0;
    }  
  } else {
    // 引数の行番
    lineno = iexp();                          
    if (err)  return 0;
    lp = getlp(lineno);
    if (lineno != getlineno(lp)) {
      // 飛び先行がない
      err = ERR_ULN;
      return 0; 
    }
  }
  return lp;
}

/**
 * @brief GOTO/GOSUB
 * @param mode 0:MODE_GOTO 1:MODE_GOSUB 2:MODE_ONGOTO 3:MODE_ONGOSUB
 * @param evtlp 
 * @note GOTO 行番号|ラベル、GOSUB 行番号|ラベル
 */
void iGotoGosub(uint8_t mode, uint16_t evtlp) {
  uint8_t* lp;

  if (mode == MODE_ONGOTO || mode == MODE_ONGOSUB)
    lp = (uint8_t*)evtlp;
  else 
    lp = getJumplp();     // 飛び先行ポインタ

  if (err)
    return;
    
  if (mode == MODE_GOSUB || mode == MODE_ONGOSUB) {
    //ポインタを退避
    if (gstki > SIZE_GSTK - 2) { // もしGOSUBスタックがいっぱいなら
      err = ERR_GSTKOF;          // エラー番号をセット
      return; 
    }
    gstk[gstki++] = clp;         // 行ポインタを退避
    gstk[gstki++] = cip;         // 中間コードポインタを退避
  }
  clp = lp;                      // 行ポインタを分岐先へ更新
  cip = clp + 3;                 // 中間コードポインタを先頭の中間コードに更新
}
