/**
 * @file idate.cpp
 * @author your name (you@domain.com)
 * @brief 日時のルーチン
 * @version 0.1
 * @date 2022-02-10
 * 
 * @copyright Copyright (c) 2022
 * 
 */

#include <Arduino.h>
#include <stdint.h>
#include "basic.h"
#include "getparam.h"
#include "intercode.h"
#include "table.h"
#include "sub.h"
#include "charinout.h"
#include "errcode.h"
#include "console.h"
#include "ttconfig.h"
#include "idate.h"

#if USE_RL78_RTC == 1
#include "RTC.h"
RTC rtc;

#define BCD(c) (uint8_t)((c/10)*16+(c%10))
// SETDATEコマンド  SETDATE 年,月,日,曜日,時,分,秒
void isetDate() {
    int16_t t[7];

  if ( getParamRange( &t[0] , 2000,2099, true) ) return; // 年
  if ( getParamRange( &t[1],    1,  12, true) ) return; // 月
  if ( getParamRange( &t[2],      1,  31, true) ) return; // 日
  if ( getParamRange( &t[3],     1,  7 , true) ) return; // 曜日の元データ
  if ( getParamRange( &t[4],     0,  23, true) ) return; // 時
  if ( getParamRange( &t[5],   0,  59, true) ) return; // 分
  if ( getParamRange( &t[6],   0,  59, false)) return; // 秒  

  int16_t year   = t[0];
  int16_t month  = t[1];
  int16_t day    = t[2];
  int16_t hour   = t[4];
  int16_t minute = t[5];
  int16_t second = t[6];
  int16_t  week  = t[3];
  int  week_flag;
  
  switch(week) {
      case 1: week_flag = RTC_WEEK_SUNDAY;     break;
      case 2: week_flag = RTC_WEEK_MONDAY;     break;
      case 3: week_flag = RTC_WEEK_TUESDAY;    break;
      case 4: week_flag = RTC_WEEK_WEDNESDAY;  break;
      case 5: week_flag = RTC_WEEK_THURSDAY;   break;
      case 6: week_flag = RTC_WEEK_FRIDAY;     break;
      case 7: week_flag = RTC_WEEK_SATURDAY;   break;
      default: week_flag = RTC_WEEK_SUNDAY; break;
  }
  // RTCの設定

    rtc.begin();

    if (rtc.setDateTime(year, month, day, hour, minute, second, week_flag) == false) {
        err= ERR_SYS;       // "Internal error"
    }
}

// 該当変数に値をセット
void setValueTo(uint16_t* rcv,uint8_t n) {
  int16_t index;  
  for (uint8_t i=0; i <n; i++) {    
    if (*cip == I_VAR) {          // 変数の場合
      cip++; index = *cip;        // 変数インデックスの取得
      var[index] = rcv[i];        // 変数に格納
      cip++;
    } else if (*cip == I_ARRAY) { // 配列の場合
      cip++;
      index = getparamValue();         // 添え字の取得
      if (err) return;  
      if (index >= SIZE_ARRY || index < 0 ) {
         err = ERR_SOR;
         return; 
      }
      arr[index] = rcv[i];        // 配列に格納
    } else {
      err = ERR_SYNTAX;           // 変数・配列でない場合はエラーとする
      return;   
    }     
    if(i != n-1) {
      if (*cip != I_COMMA) {      // ','のチェック
         err = ERR_SYNTAX;
         return; 
      }
      cip++;
    }
  }
}

int16_t getConvertedWeek(int16_t inWeek)
{
    int16_t outWeek = 0;

    switch (inWeek)  {
        case RTC_WEEK_SUNDAY:       outWeek = 1; break;
        case RTC_WEEK_MONDAY:       outWeek = 2; break;
        case RTC_WEEK_TUESDAY:      outWeek = 3; break;
        case RTC_WEEK_WEDNESDAY:    outWeek = 4; break;
        case RTC_WEEK_THURSDAY:     outWeek = 5; break;
        case RTC_WEEK_FRIDAY:       outWeek = 6; break;
        case RTC_WEEK_SATURDAY:     outWeek = 7; break;
    }
    return outWeek;
}

// GETDATEコマンド  GETDATE 年格納変数,月格納変数, 日格納変数, 曜日格納変数
void igetDate() {
  int year, month, day, hour, min, sec, week ;
  uint16_t rcv[4];
//   if (readRTC(rcv,3,4))
//     return;
//   rcv[0]+=2000;
//   setValueTo(rcv, 4);
    if (rtc.getDateTime(year, month, day, hour, min, sec, week) == true)  {
        rcv[0] = year + 2000 ;
        rcv[1] = month ;
        rcv[2] = day ;
        rcv[3] = getConvertedWeek(week) ;
        setValueTo(rcv, 4);
    }
}

// GETDATEコマンド  SETDATE 時格納変数,分格納変数, 秒格納変数
void igetTime() {
  int year, month, day, hour, min, sec, week ;
  uint16_t rcv[3];
/*   if (readRTC(rcv,0,3))
    return;
  setValueTo(rcv, 3); */
    if (rtc.getDateTime(year, month, day, hour, min, sec, week) == true)  {
        rcv[0] = hour ;
        rcv[1] = min ;
        rcv[2] = sec ;
        setValueTo(rcv, 3);
    }
}

// DATEコマンド
//KW(w1,"Sun");KW(w2,"Mon");KW(w3,"Tue");KW(w4,"Wed");KW(w5,"Thr");KW(w6,"Fri");KW(w7,"Sat");

//const char*  const weekstr[] PROGMEM = {w1,w2,w3,w4,w5,w6,w7};
const char* const weekstr[] = {"Sun","Mon","Tue","Wed","Thr","Fri","Sat"};
void idate(uint8_t devno) {

    int year, month, day, hour, min, sec, week;
    if (rtc.getDateTime(year, month, day, hour, min, sec, week) == true)  {
        putnum((int16_t)year+2000, -4 ,devno);          // rcv[0]
        c_putch('/',devno);
        putnum((int16_t)month, -2,devno);               // rcv[1]
        c_putch('/',devno);
        putnum((int16_t)day, -2,devno);                 // rcv[2]
        c_puts_devno((const char*)F(" ["),devno);
        c_puts_devno((const char*)pgm_read_word(&weekstr[week]),devno);     // rcv[3]
        c_puts_devno((const char*)F("] "),devno);
        putnum((int16_t)hour, -2,devno);                // rcv[4]
        c_putch(':',devno);
        putnum((int16_t)min, -2,devno);              // rcv[5]
        c_putch(':',devno);
        putnum((int16_t)sec, -2,devno);              // rcv[6]
        newline_dev(devno);
    }
}


// DATE$()
void idatestr(uint8_t devno) {
  if (checkOpen()) return;
  if (checkClose()) return;
  idate(devno);    
}
#endif


