/**
 * @file iprint.cpp
 * @author your name (you@domain.com)
 * @brief PRINT文の処理
 * @version 0.1
 * @date 2022-02-09
 * 
 * @copyright Copyright (c) 2022
 * 
 */

#include <stdint.h>
#include "Arduino.h"
#include "basic.h"
#include "intercode.h"
#include "charinout.h"
#include "getparam.h"
#include "console.h"
#include "gpio.h"
#include "errcode.h"
#include "sub.h"
#include "iprint.h"

/**
 * @brief PRINT文の処理
 * @param devno  出力デバイス番号
 * @param nonewln 0以外で改行出力
 * @todo OP_TESTのための修正を追加
 */
void iprint(uint8_t devno, uint8_t nonewln) {
  int16_t value;     // 値
  int16_t len = 0;   // 桁数
  uint8_t i;         // 文字数

  // 文末まで繰り返す
  while (*cip != I_COLON && *cip != I_EOL && *cip != I_ELSE) {
    switch (*cip++) { // 中間コードで分岐
    case I_STR:       // 文字列
      i = *cip++;     // 文字数を取得
      while (i--)     // 文字数だけ繰り返す
        c_putch(*cip++, devno); //文字を表示
      break; 
    case I_SHARP:   len = iexp();    break; // 「#」 桁数を取得
    case I_CHR:     ichr(devno);     break; // CHR$()関数
    case I_HEX:     ihexbin(devno,0);break; // HEX$()関数
    case I_BIN:     ihexbin(devno,1);break; // BIN$()関数     
#if USE_DMP == 1
    case I_DMP:     idmp(devno);     break; // DMP$()関数
#endif
    case I_STRREF:  iwstr(devno);    break; // STR$()関数
#if USE_I2CEEPROM == 1 && USE_CMD_I2C == 1 && USE_RTC_DS3231 == 1 || USE_RL78_RTC == 1
    case I_DATESTR: idatestr(devno); break; // DATE$()関数
#endif
    default: // 以上のいずれにも該当しなかった場合（式とみなす）
      cip--;
      value = iexp();             // 値を取得
      if (!err) 
         putnum(value, len,devno); // 値を表示
       break;
    } // 中間コードで分岐の末尾
    
    if (err)
       break;
    if (nonewln && *cip == I_COMMA)  // 文字列引数流用時はここで終了
       return;
    if (*cip == I_COMMA || *cip == I_SEMI) { // もし',' ';'があったら
       cip++;
       if (*cip == I_COLON || *cip == I_EOL || *cip == I_ELSE) //もし文末なら
          return; 
    } else if (*cip != I_COLON && *cip != I_EOL && *cip != I_ELSE) {
      //もし文末でなければ
      err = ERR_SYNTAX;
      break;
    }
  }
  if (!nonewln) {
    newline_dev(devno);
  }
}
