/** @file keyword.h
 * @brief BASIC言語のキーワード定義
 */

#ifndef __keyword_h__
#define __keyword_h__

#include "ttconfig.h"

//*** キーワードテーブル ***************************
// ※並び位置が中間コードと一致させることに注意
// ※中間コードは basic.hに定義
const char* const kwtbl[] = {
  "GOTO", "GOSUB", "RETURN", "END",
  "FOR", "TO", "STEP", "NEXT",
  "IF", "ELSE", "REM",
  "INPUT", "PRINT", "?", "LET",
  "," , ";", ":", "\'",
  "-", "+", "(", ")", "$", "`",
  "*", "/", "%", "<<", ">>", "&", "|", "^",                 // 2因子演算子:
  "=", "!=", "<>", "<", "<=", ">", ">=", "AND", "OR",      // 条件判定演算子:
  "#",
  "!", "~",                                          
  "@","RND","ABS","FREE",

  // システムコマンド 
  "RUN","LIST","RENUM","DELETE","NEW","LOAD","SAVE",
  // 下記はシステムコマンドが除外(2019/09/04)
  "ERASE","FILES","FORMAT","DRIVE",
  "CLS",

#if USE_CMD_VFD == 1 || USE_ALL_KEYWORD == 1
  "VMSG","VCLS","VSCROLL","VBRIGHT","VDISPLAY","VPUT",
#endif
  "WAIT",
  "CHR$","HEX$","BIN$","STR$",
  "BYTE","LEN","ASC",
  "COLOR","ATTR","LOCATE","INKEY",
  "GPIO", "OUT", "PWM",
  "IN", "ANA",
  "TONE","NOTONE",
#if USE_CMD_PLAY == 1 || USE_ALL_KEYWORD == 1
  "PLAY","TEMPO",
#endif
  "SYSINFO",
  // 仮想アドレス
  "VAR", "ARRAY", "PRG", "MEM", "MEM2",

  "PEEK","POKE","I2CW","I2CR","TICK",
  "MAP","GRADE","SHIFTOUT","PULSEIN",
  "DMP$","SHIFTIN",
 
// 定数  
  "OUTPUT", "PULLUP", "FLOAT",              // GPIOモード
  "OFF", "ON", "LOW", "HIGH",               // ビット状態
  "LSB","MSB",                              // ビット情報
  "UP", "DOWN", "RIGHT", "LEFT",            // キーボードコード
  "SPACE", "ENTER",                         // キーボードコード
  "CW", "CH",                               // 画面サイズ
  "Change", "Falling", "Rising",            // ピン変化
  "LED",

#if USE_RTC_DS3231 == 1 && USE_CMD_I2C == 1 || USE_ALL_KEYWORD == 1 || USE_RL78_RTC == 1
  "DATE", "GETDATE", "GETTIME", "SETDATE",
  "DATE$",
#endif
#if USE_SO1602AWWB == 1 && USE_CMD_I2C == 1 || USE_ALL_KEYWORD == 1
  "CPRINT", "CCLS", "CCURS", "CLOCATE",
  "CCONS", "CDISP",
#endif
#if USE_ANADEF == 1 || USE_ALL_KEYWORD == 1
  "A0", "A1", "A2", "A3", "A4",
  "A5","A6","A7",
  #ifdef ARDUINO_AVR_MEGA2560
    "A8","A9","A10","A11","A12",
    "A13","A14","A15",
  #endif
#endif
#if USE_IR == 1 || USE_ALL_KEYWORD == 1
  "IR",
#endif
// 美咲フォントの利用
#if USE_MISAKIFONT != 0 || USE_ALL_KEYWORD == 1
  "GetFont",
#endif
// NeoPixelの利用
#if USE_NEOPIXEL == 1 || USE_ALL_KEYWORD == 1

  "NInit", "NBright", "NCls", "NSet",
  "NPset", "NMsg", "NUpdate", "NShift",
  "RGB", "NLine", "NScroll", "NPoint",

#endif
// イベントの利用
#if USE_EVENT == 1 || USE_ALL_KEYWORD == 1
  "Timer", "Pin", "Sleep",
#endif
// キャラクタディスプレイ（LCD 4bit I/F)
#if USE_SC1602BS == 1 || USE_ALL_KEYWORD == 1
  "LPrint", "LCls", "LCurs", "LLocate",
  "LCons", "LDisp",
#endif
  "OK",                                          
};

#define SIZE_KWTBL (sizeof(kwtbl) / sizeof(kwtbl[0]))

#endif
