/** @file table.h
 *  @brief 各種テーブル
 */

#ifndef __table_h__
#define __table_h__

#include "ttconfig.h"

//*** 定数 ***************************************
#define CONST_HIGH   1   // HIGH
#define CONST_LOW    0   // LOW
#define CONST_ON     1   // ON
#define CONST_OFF    0   // OFF
#define CONST_LSB    0   // LSB
#define CONST_MSB    1   // MSB

//*** 仮想メモリ定義 ******************************
#define V_VRAM_TOP  0x0000    // VRAM領域先頭
#define V_VAR_TOP   0x1900    // 変数領域先頭
#define V_ARRAY_TOP 0x1AA0    // 配列領域先頭getParam
#define V_PRG_TOP   0x1BA0    // プログラム領域先頭
#define V_MEM_TOP   0x2BA0    // ユーザー作業領域先頭
#define V_MEM2_TOP  0x2CA0    // ユーザー作業領域2先頭

//*** LIST出力整形用定義テーブル ********************
// 後ろに空白を入れない中間コード
const unsigned char i_nsa[] = {
  I_RETURN, I_COMMA, I_SEMI, I_COLON,I_END,
  I_MINUS, I_PLUS, I_MUL, I_DIV,  I_DIVR, I_OPEN, I_CLOSE, I_DOLLAR, I_APOST,
  I_LSHIFT, I_RSHIFT, I_OR, I_AND, I_NEQ, I_NEQ2, I_XOR,
  I_GTE, I_SHARP, I_GT, I_EQ, I_LTE, I_LT, I_LNOT, I_BITREV, I_DIVR,
  I_ARRAY, I_RND, I_ABS, I_SIZE,I_CLS, I_QUEST,
  I_CHR, I_HEX, I_BIN,I_STRREF,
#if USE_RTC_DS3231 == 1 && USE_CMD_I2C == 1 || USE_ALL_KEYWORD == 1 || USE_RL78_RTC == 1
  I_DATESTR,
#endif
#if USE_CMD_VFD == 1 || USE_ALL_KEYWORD == 1
  I_VCLS,
#endif
  I_INKEY,
  I_LEN, I_BYTE, I_ASC,
  I_OUTPUT, I_INPUT_PU,I_INPUT_FL,
  I_OFF, I_ON, I_DIN, I_ANA,
  I_SYSINFO,
  I_MEM, I_MVAR, I_MARRAY,I_MPRG, I_MEM2, 
  I_PEEK, I_I2CW, I_I2CR, I_TICK,
  I_MAP, I_GRADE, I_SHIFTIN, I_PULSEIN, I_DMP,
  I_KUP, I_KDOWN, I_KRIGHT, I_KLEFT, I_KSPACE, I_KENTER,  // キーボードコード
  I_LSB, I_MSB,I_CW, I_CH,  
#if USE_ANADEF == 1 || USE_ALL_KEYWORD == 1
  I_A0,I_A1,I_A2,I_A3,I_A4,I_A5,I_A6,I_A7,
  #ifdef ARDUINO_AVR_MEGA2560
    I_A8,I_A9,I_A10,I_A11,I_A12,I_A13,I_A14,I_A15,
  #endif  
#endif
#if USE_IR == 1 || USE_ALL_KEYWORD == 1
  I_IR,
#endif
#if USE_MISAKIFONT != 0 || USE_ALL_KEYWORD == 1
I_GETFONT,
#endif
#if USE_NEOPIXEL == 1 || USE_ALL_KEYWORD == 1
 I_RGB, I_NPOINT,
#endif
};

// 前が定数か変数のとき前の空白をなくす中間コード
const unsigned char i_nsb[] = {
  I_RETURN, I_COMMA,I_SEMI, I_COLON,I_SQUOT,
  I_MINUS, I_PLUS, I_MUL, I_DIV,  I_DIVR,I_OPEN, I_CLOSE,I_LSHIFT, I_RSHIFT, I_OR, I_AND,
  I_GTE, I_SHARP, I_GT, I_EQ, I_LTE,  I_NEQ, I_NEQ2,I_LT, I_LNOT, I_BITREV, I_XOR,
  I_ARRAY, I_RND, I_ABS, I_SIZE  
};

// 必ず前に空白を入れる中間コード
const unsigned char i_sf[]  = {
  I_ATTR, I_CLS, I_COLOR, I_END, I_FILES, I_TO, I_STEP,I_QUEST,I_LAND, I_LOR,
  I_GOSUB,I_GOTO,I_INPUT,I_LET,I_LIST,I_ELSE,I_RENUM,
  I_LOAD,I_LOCATE,I_NEW,I_PRINT,
  I_RETURN,I_RUN,I_SAVE,I_WAIT,
#if USE_CMD_VFD == 1 || USE_ALL_KEYWORD == 1
  I_VMSG, I_VPUT,
#endif
  I_GPIO, I_DOUT,I_RENUM,
  I_TONE, I_NOTONE,I_SYSINFO,I_POKE,
#if USE_CMD_PLAY == 1 || USE_ALL_KEYWORD == 1
  I_PLAY, I_TEMPO,
#endif
#if USE_RTC_DS3231 == 1 && USE_CMD_I2C == 1 || USE_ALL_KEYWORD == 1 || USE_RL78_RTC == 1
  I_DATE, I_GETDATE, I_GETTIME, I_SETDATE,   // RTC関連コマンド(4)  
#endif 
  I_FORMAT,I_DRIVE,
#if USE_SO1602AWWB == 1 && USE_CMD_I2C == 1 || USE_ALL_KEYWORD == 1
  I_CPRINT, I_CCLS, I_CCURS, I_CLOCATE, I_CCONS, I_CDISP,  
#endif
#if USE_NEOPIXEL == 1 || USE_ALL_KEYWORD == 1
  I_NINIT, I_NBRIGHT, I_NCLS, I_NSET, I_NPSET, I_NMSG, I_NUPDATE, I_NSHIFT, I_NLINE,I_NSCROLL,
#endif
#if USE_EVENT == 1 || USE_ALL_KEYWORD == 1
  I_TIMER, I_PIN,
#endif
#if USE_SC1602BS == 1 || USE_ALL_KEYWORD == 1
  I_LPRINT, I_LCLS, I_LCURS, I_LLOCATE, I_LCONS, I_LDISP,
#endif
};

// 後ろが変数、数値、定数の場合、後ろに空白を空ける中間コード
const uint8_t i_sb_if_value[] = {
  I_NUM, I_STR, I_HEXNUM, I_VAR, I_BINNUM,
  I_OFF, I_ON, I_MEM, I_MVAR, I_MARRAY,I_MPRG, I_MEM2, 
  I_KUP, I_KDOWN, I_KRIGHT, I_KLEFT, I_KSPACE, I_KENTER,  // キーボードコード
  I_LSB, I_MSB,I_CW, I_CH,  
#if USE_ANADEF == 1 || USE_ALL_KEYWORD == 1
  I_A0,I_A1,I_A2,I_A3,I_A4,I_A5,I_A6,I_A7,
  #ifdef ARDUINO_AVR_MEGA2560
    I_A8,I_A9,I_A10,I_A11,I_A12,I_A13,I_A14,I_A15,
  #endif 
#endif
};

// 定数テーブル
const uint8_t constValue[] = {
  OUTPUT,INPUT_PULLUP,INPUT,
  CONST_OFF,CONST_ON,CONST_LOW,CONST_HIGH,CONST_LSB,CONST_MSB,
  0x1c,0x1d,0x1e,0x1f,0x20,0x0d,
  c_getWidth(),c_getHeight(),
  1,2,3,
  13,
};

// 仮想アドレステーブル  
// @todo realadress.cpp に移動する
static const uint16_t vatable [] = {
  V_VAR_TOP, V_ARRAY_TOP,  V_PRG_TOP, V_MEM_TOP, V_MEM2_TOP,  
};

#endif
