//
// Arduino Uno互換機+「アクティブマトリクス蛍光表示管（CL-VFD）MW25616L 実験用表示モジュール」対応
// 豊四季Tiny BASIC for Arduino uno 構築コンフィグレーション
// 修正 2019/06/11 GETFONTコマンド利用オプション設定の追加（美咲フォント対応）
// 修正 2019/07/13 NeoPixel、タイマーイベント利用オプション設定の追加
// 修正 2019/08/04 外部割込みイベント利用オプション設定の追加
// 修正 2019/09/07 機能利用オプション設定のデフォルト設定の見直し
// 修正 2019/10/08 MEGA2560用の機能利用オプション設定を追加
// @todo 2020/07/12 USE_GRADE USE_DMP をセットしてビルドしたが動作確認はしてない
// @todo 2020/07/15 USE_CMD_I2 等をセットしてビルドしたが動作確認はしてない
// @todo 2020/07/15 USE_CMD_PLAYでエラー関数が見つからない

#ifndef __ttconfig_h__
#define __ttconfig_h__

#define MYDEBUG 0

/*
 * デフォルトの設定は Arduino UNO 用の設定になっています。
 * 「アクティブマトリクス蛍光表示管（CL-VFD）MW25616L 実験用表示モジュール」
 * で利用する場合は下記の変更を行って下さい。
 *   USE_CMD_VFD  0 ==> 1
 * また、ローカル変数領域不足により動作が不安定になる場合があります。
 * その場合は、プログラム領域サイズを512～1024の間で調整して下さい。
 */

// ** シリアルポート通信速度 *************************************************
#define SERIALBAUD 115200
//#define SERIALBAUD 9800

// ** 利用ピン設定 **********************************************************

#ifdef GRADZUKI
  // GR-ADZUKI
  #define   TonePin 21  // Tone用出力ピン（圧電スピーカー接続）(A7)
  #define   AutoPin 7  // 自動起動チェックピン
#endif
// ** プログラム領域サイズ ***************************************************

#ifdef GRADZUKI
  // RL78は1ブロック 1024バイトで管理しているので、1024にしてある。 GR-AZUKIは8ブロックで構成される。
  //（重要） pfdlライブラリの使用を前提に 1ブロック 1024と仮定しているので、他の値は指定できない。
  // アドレスは、ブロックの先頭からのオフセット(0-8191)であるが、1024ごとに境界がある。
  // 例えば　1020-1030まで書き込むときは 1020-1023, 1024-1030 と2回に分けて読み書きする必要がある
  // 境界をまたいで書き込みはできないので管理を簡単にするため PRGARESIZE 1024とする

  #define   PRGAREASIZE 2048 // プログラム領域サイズ(データフラッシュを使う場合、現状では変更できない）
  #define   ARRYSIZE    100  // 配列領域
  #define   FLASHBLOCKSIZE 1024 // 内部データフラッシュの1ブロックのサイズ 1024バイト
#endif
#define USE_ALL_KEYWORD  1   // 未使用キーワードも有効にする(1:有効 2:無効 デフォルト:1)

#ifdef GRADZUKI
// ** 機能利用オプション設定 for Arduino Uno *********************************
#define USE_CMD_PLAY   1  // PLAYコマンドの利用(0:利用しない 1:利用する デフォルト:0)
#define USE_CMD_I2C    1  // I2Cコマンドの利用(0:利用しない 1:利用する デフォルト:1)
#define USE_PULSEIN    1  // PULSEIN関数の利用(0:利用しない 1:利用する デフォルト:1)
#define USE_SHIFTIN    1  // SHIFTIN関数の利用(0:利用しない 1:利用する デフォルト:1)
#define USE_SHIFTOUT   1  // SHIFTOUTコマンドの利用(0:利用しない 1:利用する デフォルト:1)
#define USE_CMD_VFD    0  // VFDモジュールコマンドの利用(0:利用しない 1:利用する デフォルト:0)
#define USE_RTC_DS3231 0  // I2C接続RTC DS3231の利用(0:利用しない 1:利用する デフォルト:0)
#define USE_I2CEEPROM  1  // I2C EEPROM対応(0:利用しない 1:利用する デフォルト:1)
#define USE_SYSINFO    1  // SYSINFOコマンド(0:利用しない 1:利用する デフォルト:1)
#define USE_GRADE      1  // GRADE関数(0:利用しない 1:利用する デフォルト:1)
#define USE_DMP        1  // DMP$関数(0:利用しない 1:利用する デフォルト:1)
#define USE_IR         1  // IR関数(0:利用しない 1:利用する デフォルト:1)
#define USE_ANADEF     1  // アナログピン定数A0～A7orA15(0:利用しない 1:利用する デフォルト:1)
#define USE_SO1602AWWB 0  // 有機ELキャラクタディスプレイ SO1602AWWB(0:利用しない 1:利用する デフォルト:0)
#define USE_MISAKIFONT 0  // 美咲フォント500文字の利用(0:利用しない 1:利用する 2:非漢字のみ利用 デフォルト:0)
#define USE_NEOPIXEL   0  // NeoPixelの利用(0:利用しない 1:利用する デフォルト:0)
#define USE_EVENT      1  // タイマー・外部割込みイベントの利用(0:利用しない 1:利用する デフォルト:1)
#define USE_SLEEP      0  // SLEEPコマンドの利用(0:利用しない 1:利用する デフォルト:1) ※USE_EVENTを利用必須
#define USE_CMD_TONE   0  // TONEコマンドの利用（0:利用しない 1:利用する デフォルト:0) RL78で追加   
#define USE_RL78_RTC   1  // GR-ADZUKIの時計機能関数の利用(0:利用しない 1:利用する デフォルト:1)
#define USE_HS_EEPROM  1  // 内部EEPROMのブロックリード／ライト機能を使用する(0:利用しない 1:利用する デフォルト:0)
#define USE_SC1602BS   0  // 4bit接続 LCDキャラクタディスプレイ(0:利用しない 1:利用する デフォルト:0)
#define USE_TEST_OP    0  // HOSTからテストしやすいようにCURSUSをPASSする　（1:利用する デフォルト:0)
#endif

#endif
